@@ -9,8 +9,8 @@ import numpy as np
 from tqdm import tqdm
 from warnings import warn
 from typing import Union
+from .patch.base import PatchInferencerBase
 from tempfile import mktemp
-
 from chunkflow.chunk import Chunk
 # from chunkflow.chunk.affinity_map import AffinityMap
 
@@ -31,7 +31,7 @@ class Inferencer(object):
     large chunk size.
     """
     def __init__(self,
-                 convnet_model: str,
+                 convnet_model: Union[str, PatchInferencerBase],
                  convnet_weight_path: str,
                  input_patch_size: Union[tuple, list],
                  output_patch_size: Union[tuple, list] = None,
@@ -170,6 +170,12 @@ class Inferencer(object):
         self._construct_output_chunk_mask(input_chunk)
 
     def _prepare_patch_inferencer(self, framework, convnet_model, convnet_weight_path, bump):
+        
+        # allow to pass patch_inferencer directly, if so assign and return
+        if framework == 'prebuilt':
+            self.patch_inferencer = convnet_model
+            return
+
         # prepare for inference
         if framework == 'pznet':
             from .patch.pznet import PZNet as PatchInferencer
@@ -268,8 +274,8 @@ class Inferencer(object):
             #                                   shape=self.output_size)
             output_mask_array = np.zeros(self.output_size, self.dtype)
         else:
-            output_chunk_mask_array = self.output_chunk_mask.array
-            output_chunk_mask_array.fill(0)
+            output_mask_array = self.output_chunk_mask.array
+            output_mask_array.fill(0)
 
         output_voxel_offset = tuple(io + ocso for io, ocso in zip(
             input_chunk.voxel_offset, self.output_offset))
@@ -314,19 +320,17 @@ class Inferencer(object):
             voxel_offset=output_voxel_offset,
             voxel_size=input_chunk.voxel_size
         )
-        assert output_buffer == 0
         return output_buffer
 
-    def __call__(self, input_chunk: np.ndarray):
+    def __call__(self, input_chunk: Chunk):
         """
         args:
-            input_chunk (Chunk): input chunk with global offset
+            input_chunk (Chunk): input chunk with voxel offset and voxel size 
         """
         assert isinstance(input_chunk, Chunk)
         
         self._update_parameters_for_input_chunk(input_chunk)
         output_buffer = self._get_output_buffer(input_chunk)
-
         if not self.mask_output_chunk:
             self._check_alignment()
          
@@ -341,7 +345,8 @@ class Inferencer(object):
             return Chunk.create(
                 size=size,
                 dtype = output_buffer.dtype,
-                voxel_offset=output_buffer.voxel_offset
+                voxel_offset=output_buffer.voxel_offset,
+                voxel_size=input_chunk.voxel_size,
             )
        
         if input_chunk == 0:
@@ -355,7 +360,8 @@ class Inferencer(object):
         if np.issubdtype(input_chunk.dtype, np.integer):
             # normalize to 0-1 value range
             dtype_max = np.iinfo(input_chunk.dtype).max
-            input_chunk = input_chunk.astype(self.dtype) / dtype_max
+            input_chunk = input_chunk.astype(self.dtype)
+            input_chunk /= dtype_max
 
         chunk_time_start = time.time()
 
@@ -391,8 +397,10 @@ class Inferencer(object):
                 # the slices[0] is for input patch slice
                 # the slices[1] is for output patch slice
                 offset = tuple(s.start for s in slices[1])
-                output_chunk = Chunk(output_patch[batch_idx, :, :, :, :],
-                                     voxel_offset=offset)
+                output_chunk = Chunk(
+                    output_patch[batch_idx, :, :, :, :],
+                    voxel_offset=offset,
+                    voxel_size=input_chunk.voxel_size)
 
                 ## save some patch for debug
                 #bbox = output_chunk.bbox
