@@ -17,6 +17,7 @@ from cloudvolume.lib import Vec
 
 from chunkflow.lib.aws.sqs_queue import SQSQueue
 from chunkflow.lib.bounding_boxes import BoundingBox, BoundingBoxes
+from chunkflow.lib.synapses import Synapses
 
 from chunkflow.chunk import Chunk
 from chunkflow.chunk.affinity_map import AffinityMap
@@ -102,8 +103,9 @@ def generate_tasks(
         aligned_block_size=aligned_block_size
     )
     # if state['verbose']:
-    print('total number of tasks: ', len(bboxes)) 
-
+    bbox_num = len(bboxes)
+    print('total number of tasks: ', bbox_num) 
+    
     if task_index_start:
         if task_index_stop is None:
             task_index_stop = task_index_start + 1
@@ -126,10 +128,11 @@ def generate_tasks(
         queue = SQSQueue(queue_name)
         queue.send_message_list(bboxes)
     else:
-        bbox_num = len(bboxes)
         for bbox_index, bbox in enumerate(bboxes):
-            if state['verbose']:
-                print(f'executing task {bbox_index} in {bbox_num}...')
+            if disbatch:
+                assert len(bboxes) == 1
+                bbox_index = disbatch_index
+            print(f'executing task {bbox_index} in {bbox_num} with bounding box: {bbox.to_filename()}')
             task = get_initial_task()
             task['bbox'] = bbox
             task['bbox_index'] = bbox_index
@@ -143,15 +146,15 @@ def generate_tasks(
     help='the pre part of result file path')
 @click.option('--post', '-t', required=True, type=str,
     help='the post part of result file path. Normally include file extention.')
-@click.option('--grow-size', '-g', default=None, type=int, callback=default_none,
+@click.option('--adjust-size', '-a', default=None, type=int, callback=default_none,
     help='expand or shrink the bounding box. Currently, cloud-volume Bbox only support symetric grow.')
 @operator
-def skip_task(tasks: Generator, pre: str, post: str, grow_size: int):
+def skip_task(tasks: Generator, pre: str, post: str, adjust_size: int):
     """if a result file already exists, skip this task."""
     for task in tasks:
         bbox = task['bbox'].clone()
-        if grow_size is not None:
-            bbox.grow(grow_size)
+        if adjust_size is not None:
+            bbox.grow(adjust_size)
         file_name = pre + bbox.to_filename() + post
         if os.path.exists(file_name):
             print('the result file already exist, skip this task')
@@ -166,7 +169,7 @@ def skip_task(tasks: Generator, pre: str, post: str, grow_size: int):
     help = 'pre-path of a file. we would like to keep a trace that this task was executed.')
 @click.option('--post', '-t', type=str, default=None,
     help='post-path of a file. normally include the extention of result file.')
-@click.option('--adjust-size', '-g', type=int, default=None,
+@click.option('--adjust-size', '-a', type=int, default=None,
     help='change the bounding box of chunk if it do not match with final result file name.')
 @operator
 def skip_all_zero(tasks, input_chunk_name: str, pre: str, post: str, adjust_size: int):
@@ -178,11 +181,7 @@ def skip_all_zero(tasks, input_chunk_name: str, pre: str, post: str, adjust_size
                 print('all zero chunk, skip this task')
                 if pre is not None:
                     bbox = chunk.bbox.clone()
-                    if adjust_size is not None:
-                        # bbox.grow(grow_size)
-                        # currently, cloud-volume do not support negative grow size
-                        bbox.minpt -= adjust_size
-                        bbox.maxpt += adjust_size
+                    bbox.adjust(adjust_size)
                     fname = os.path.join(pre, f'{bbox.to_filename()}{post}')
                     print('create an empty file as mark: ', fname)
                     with open(fname, 'a'):
@@ -192,6 +191,21 @@ def skip_all_zero(tasks, input_chunk_name: str, pre: str, post: str, adjust_size
         yield task
 
 
+@main.command('skip-none')
+@click.option('--input-name', '-i',
+    type=str, default=DEFAULT_CHUNK_NAME, help='input name')
+@operator
+def skip_none(tasks, input_name: str):
+    """If item is None, skip this task."""
+    for task in tasks:
+        if task is not None:
+            data = task[input_name]
+            if data is None:
+                # label task as None and task will be skipped
+                task = None
+        yield task
+
+
 @main.command('setup-env')
 @click.option('--volume-start', required=True, nargs=3, type=int,
               help='start coordinate of output volume in mip 0')
@@ -490,23 +504,97 @@ def create_chunk(tasks, name, size, dtype, all_zero, voxel_offset, voxel_size, o
                 voxel_size=voxel_size)
         yield task
 
+
+@main.command('load-synapses')
+@click.option('--name', '-n', type=str, default='load-synapses', help='name of operator')
+@click.option('--file-path', '-f',
+    type=click.Path(file_okay=True, dir_okay=True, resolve_path=True),
+    required=True, help='files containing synapses. Currently support HDF5 and JSON.')
+@click.option('--path-suffix', '-s', type=str, default=None, help='file path suffix.')
+@click.option('--resolution', '-r', type=int, nargs=3, 
+    default=None, callback=default_none, help='resolution of points.')
+@click.option('--output-name', '-o', type=str, default='synapses', help='data name of the result.')
+@operator
+def load_synapses(tasks, name: str, file_path: str, path_suffix: str, resolution: tuple, output_name: str):
+    """Load synapses formated as JSON or HDF5."""
+    for task in tasks:
+        if task is not None:
+            start = time()
+            if os.path.isdir(file_path):
+                bbox = task['bbox']
+                assert path_suffix is not None
+                fname = os.path.join(file_path, f'{bbox.to_filename()}{path_suffix}')
+            elif not os.path.exists(file_path):
+                bbox = task['bbox']
+                fname = f'{file_path}{bbox.to_filename()}{path_suffix}'
+            else:
+                fname = file_path
+            assert os.path.exists(fname)
+
+            if os.path.getsize(fname) == 0:
+                task[output_name] = None
+            else:
+                task[output_name] = Synapses.from_file(fname, resolution = resolution)
+            
+            task['log']['timer'][name] = time() - start
+        yield task
+
+
+@main.command('read-npy')
+@click.option('--name', '-n', type=str, default='read-npy', help='name of operator')
+@click.option('--file-path', '-f', 
+    type=click.Path(file_okay=True, dir_okay=True, resolve_path=True),
+    required=True, help='NPY file path')
+@click.option('--resolution', '-r', type=float, nargs=3, default=(1,1,1), help='resolution of points.')
+@click.option('--output-name', '-o', type=str, default='array', help='data name of the result.')
+@operator
+def read_npy(tasks, name: str, file_path: str, resolution: tuple, output_name: str):
+    """Read NPY files."""
+    for task in tasks:
+        if task is not None:
+            start = time()
+            if not file_path.endswith('.npy'):
+                bbox = task['bbox']
+                if os.path.isdir(file_path):
+                    file_path = os.path.join(file_path, f'{bbox.to_filename()}.npy')
+                else:
+                    file_path = f'{file_path}{bbox.to_filename()}.npy'
+            assert os.path.exists(file_path)
+            if 0 == os.path.getsize(file_path):
+                task[output_name] = None
+            else:
+                with open(file_path, 'rb') as file:
+                    array = np.load(file)
+                array *= np.asarray(resolution, dtype=array.dtype)
+                task[output_name] = array
+            task['log']['timer'][name] = time() - start
+        yield task
+
 @main.command('read-json')
 @click.option('--name', '-n', type=str, default='read-json', help='name of operator.')
-@click.option('--file-name', '-f', type=str, default='read-json', help='JSON file name')
-@click.option('--output-name', '-o', type=str, default='dict', help='output as a data name.')
+@click.option('--file-path', '-f', 
+    type=click.Path(file_okay=True, dir_okay=True, resolve_path=True), 
+    required=True, help='JSON file name')
+@click.option('--output-name', '-o', type=str, default='dict', help='data name of the result.')
 @operator
-def read_json(tasks, name: str, file_name: str, output_name: str):
+def read_json(tasks, name: str, file_path: str, output_name: str):
     """Read JSON file."""
     for task in tasks:
         if task is not None:
             start = time()
-            file_name = os.path.expanduser(file_name)
-            assert os.path.exists(file_name)
-            with open(file_name, 'r') as file:
+            if not file_path.endswith('.json'):
+                bbox = task['bbox']
+                if os.path.isdir(file_path):
+                    file_path = os.path.join(file_path, f'{bbox.to_filename()}.json')
+                else:
+                    file_path = f'{file_path}{bbox.to_filename()}.json'
+            assert os.path.exists(file_path)
+            with open(file_path, 'r') as file:
                 task[output_name] = json.load(file)
             task['log']['timer'][name] = time() - start
         yield task
 
+
 @main.command('read-nrrd')
 @click.option('--name', type=str, default='read-nrrd',
               help='read nrrd file from local disk.')
@@ -1165,8 +1253,8 @@ def plugin(tasks, name: str, input_names: str, output_names: str, file: str, arg
               type=click.Choice(['6', '18', '26']),
               default='6', help='number of neighboring voxels used. Default is 6.')
 @operator
-def connected_components(tasks, name, input_chunk_name, output_chunk_name, 
-                         threshold, connectivity):
+def connected_components(tasks, name: str, input_chunk_name: str, output_chunk_name: str, 
+                         threshold: float, connectivity: str):
     """Threshold the probability map to get a segmentation."""
     connectivity = int(connectivity)
     for task in tasks:
@@ -1174,7 +1262,7 @@ def connected_components(tasks, name, input_chunk_name, output_chunk_name,
             start = time()
             task[output_chunk_name] = task[input_chunk_name].connected_component(
                 threshold=threshold, connectivity=connectivity)
-            task['log']['timer']['name'] = time() - start
+            task['log']['timer'][name] = time() - start
         yield task
 
 
@@ -1341,7 +1429,6 @@ def mask_out_objects(tasks, input_chunk_name, output_chunk_name,
             seg = task[input_chunk_name]
             if not isinstance(seg, Segmentation):
                 assert isinstance(seg, Chunk)
-                assert seg.is_segmentation
                 seg = Segmentation.from_chunk(seg)
 
             if dust_size_threshold is not None:
@@ -1511,10 +1598,13 @@ def quantize(tasks, name, input_chunk_name, output_chunk_name):
     default=False, help='create thumbnail or not. ' +
     'the thumbnail is a downsampled and quantized version of the chunk.')
 @click.option('--intensity-threshold', '-t',
-    default=None, help='do not save anything if all voxel intensity is below threshold.'
+    default=None, type=float,
+    help='do not save anything if all voxel intensity is below threshold.'
 )
 @operator
-def write_precomputed(tasks, name, volume_path, input_chunk_name, mip, upload_log, create_thumbnail, intensity_threshold):
+def write_precomputed(tasks, name: str, volume_path: str, 
+        input_chunk_name: str, mip: int, upload_log: bool, 
+        create_thumbnail: bool, intensity_threshold: float):
     """Save chunk to volume."""
     if mip is None:
         mip = state['mip']
