@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-
+import logging
 
 from .base import OperatorBase
 from chunkflow.chunk.segmentation import Segmentation
@@ -11,8 +11,7 @@ from cloudvolume.storage import Storage
 class SkeletonizeOperator(OperatorBase):
     """Create mesh files from segmentation."""
     def __init__(self, output_path,
-                 name: str='skeletonize',
-                 verbose: int=1):
+                 name: str='skeletonize'):
         """
         Parameters
         ------------
@@ -20,10 +19,8 @@ class SkeletonizeOperator(OperatorBase):
             where to put the skeleton files
         name: 
             operator name.
-        verbose:
-            print out informations or not.
         """
-        super().__init__(name=name, verbose=verbose)
+        super().__init__(name=name)
         self.storage = Storage(output_path)
 
     def __call__(self, seg, voxel_size):
@@ -31,8 +28,7 @@ class SkeletonizeOperator(OperatorBase):
             print('no segmentation, skip computation.')
             return None
  
-        if self.verbose:
-            print('skeletonize segmentation...')
+        logging.info('skeletonize segmentation...')
       
         seg = Segmentation.from_chunk(seg)
         skels = seg.skeletonize(voxel_size)
